package nl.uvt.locator;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import nu.xom.*;

public class TucatInfo extends LinkInfo {
	String epn;
	String status;
	String callNumber;
	String material;
	String collection;
	String action;
	String year;
	String startYear;
	String endYear;
	Node holding;

	TucatInfo(){
		epn = "";
		status = "";
		callNumber =  "";
		material = "";
		action = "";
		collection = "";
		year = "";
		startYear = "";
		endYear = "";
		holding = null;
	}

	void fill( Node rec ){
		logger.debug( "fill from record:" + rec.toXML() );
		linkMethod = "tucat";
		String res = getValue( rec, "issn");
		if ( res.length() > 0 )
			linkIssn = res;
		res = getValue( rec, "title");
		if ( res.length() > 0 ){
			linkTitle = res; }
		res = getValue( rec, "url");
		if ( res.length() > 0 ){
			linkUrl = res;
		}
		res = getValue( rec, "publisher");
		if ( res.length() > 0 ){
			linkPublisher = res;
		}
		res = getValue( rec, "epn");
		if ( res.length() > 0 )
			epn = res;
		res = getValue( rec, "status");
		if ( res.length() > 0 )
			status = res;
		String[] resArr = getValues( rec, "action");
		//
		// this is a rather gross HACK to see if at least one range is passed
		//
		if ( resArr != null )
			for ( int i=0; i< resArr.length; ++i ){
				action = resArr[i];
				if ( action.equals("passed") ){
					break;
				}
			}
		res = getValue( rec, "year");
		if ( res.length() > 0 )
			year = cleanYear(res);
		res = getValue( rec, "year-range");
		if ( res.length() > 0 )
			getYearRange(res );
		res = getValue( rec, "collection");
		if ( res.length() > 0 )
			collection = res;
		res = getValue( rec, "material");
		if ( res.length() > 0 )
			material = res;
		res = getValue( rec, "callNumber");
		if ( res.length() > 0 )
			callNumber = res;
		holding = getXmlValue( rec, "holding");
	}		

	private void getYearRange(String res ) {
		String[] parts = res.split( "-" );
		if ( parts.length >= 2 ){
			startYear = cleanYear(parts[0]);
			endYear = cleanYear(parts[1]);
		}
		else if ( parts.length == 1 ){
			startYear = cleanYear(parts[0]);
			endYear = cleanYear(parts[0]);
		}
	}

	Node toRecordNode(){
		Element record = new Element( "record" );
		record.appendChild( makeNode( "linkIssn", linkIssn ) );
		record.appendChild( makeNode( "linkTitle", linkTitle ) );
		record.appendChild( makeNode( "linkUrl", linkUrl ) );
		record.appendChild( makeNode( "generatedUrl", generatedUrl ) );
		record.appendChild( makeNode( "linkMethod", linkMethod ) );
		record.appendChild( makeNode( "action", action ) );
		record.appendChild( makeNode( "collection", collection ) );
		record.appendChild( makeNode( "epn", epn ) );
		record.appendChild( makeNode( "callNumber", callNumber ) );
		record.appendChild( makeNode( "material", material ) );
		if ( holding != null )
			record.appendChild( holding );
		else
			record.appendChild( makeNode( "holding", "" ) );	
		record.appendChild( makeNode( "status", status ) );
		return record;
	}

	public boolean applyMethod( QueryData pQ ){
//		logger.debug("handle TuCat method: " + status );
		// we just return the result form our .xsl script
		logger.debug("incoming action="+action);
//		logger.debug("status="+status);
		if ( !action.equals("reject") ){
			if ( !collection.equals("") ){
				Pattern braces = Pattern.compile("\\((.+?)\\)");
				Matcher myMatch = braces.matcher(collection); 
				String coll = "";
				while ( myMatch.find()){
					coll = myMatch.group(1);
				}
				logger.debug( "final collection match = " + coll );
				collection = coll;
			}
			if ( collection.equals("") )
				collection = tryToFixCollection( callNumber );
//			logger.debug( "collection=" + collection );
//			logger.debug( "epn=" + epn );
//			logger.debug( "status=" + status );
//			logger.debug( "callNumber=" + callNumber );
			if ( pQ.genre.equals("book") && pQ.year > 0 ){
				if ( !startYear.equals("")){
					logger.debug( " vergelijk jaren '" + pQ.year + "' met range " + startYear + "-" + endYear );
					String y1 = startYear.replace('X', '0' );
					String y2 = endYear.replace('X', '0' );
					try {
						int sY = Integer.parseInt(y1);
						int eY = Integer.parseInt(y2);
						if ( pQ.year < sY || pQ.year > eY )
							action = "reject";
					}
					catch( Exception e ){
						logger.debug("problem converting year to int: " + e.getMessage());
					}
				}
				else if ( !year.equals( "" ) ){
					logger.debug( " vergelijk jaar '" + pQ.year + "' met '" + year + "'" );
					String y1 = year.replace('X', '0' );
					try {
						int sY = Integer.parseInt(y1);
						if ( pQ.year != sY )
							action = "reject";
					}
					catch( Exception e ){
						logger.debug("problem converting year to int: " + e.getMessage());
					}
				}
			}
			logger.debug("intermediate action=" + action );
			logger.debug("callNumber="+callNumber);
			logger.debug("epn="+epn);
			logger.debug("status="+status);
			if ( action.equals("") || action.equals("passed")){
//				if ( !linkUrl.equals("") ){
//					action = "url";
//					generatedUrl = linkUrl;				
//				}
//				else 
					if ( !epn.equals("") && !status.equals("") && !callNumber.equals("") ) {
					action = "passed";
				}
				else
					action = "reject";
			}
		}
		logger.debug("generatedUrl="+generatedUrl);
		logger.debug("final action="+action);
		return !action.equals("reject");
	}

	String cleanYear( String year ){
		String result = year.replace('[', ' ' );
		result = result.replace(']', ' ' );
		result = result.replace("cop.", "" );
		result = result.replace( " ", "" );
		return result;
	}

	private String tryToFixCollection(String call ) {
		//
		// try to guess the collection from the callnumber
		// rather naive
		//
		String[] knownCollections = { "BRA", "CBM", "CBC", "DISC", "ECO", "JUR", "LET", "SOC", "TFL" };
		for ( int i=0; i < knownCollections.length; ++i ){
			String tst = knownCollections[i];
			if ( call.contains( tst ))
				return tst;
		}
		return "";
	}
}
